@@ -15,7 +15,8 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured, ValidationError
 from django.core.urlresolvers import reverse
 from django.conf import settings
-from django.utils.translation import ugettext as _, ugettext_lazy as __
+from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy
 
 from wagtail.wagtailcore.models import Page
 from wagtail.wagtailcore.util import camelcase_to_underscore
@@ -78,7 +79,7 @@ class FriendlyTimeField(forms.CharField):
             return datetime.time(hour=hour, minute=minute)
         else:
             raise ValidationError(_("Please type a valid time"))
-            
+
 
 class LocalizedDateInput(forms.DateInput):
     """
@@ -127,7 +128,7 @@ class LocalizedTimeField(forms.CharField):
                 minute = 0
             if hour>=24 or hour < 0 or minute >=60 or minute < 0:
                 raise ValidationError(_("Please type a valid time"))
-                
+
             return datetime.time(hour=hour, minute=minute)
         else:
             raise ValidationError(_("Please type a valid time") )
@@ -138,7 +139,7 @@ if hasattr(settings, 'USE_L10N') and settings.USE_L10N==True:
         models.DateField: {'widget': LocalizedDateInput},
         models.TimeField: {'widget': LocalizedTimeInput, 'form_class': LocalizedTimeField},
     }
-else: # Fall back to friendly date/time            
+else: # Fall back to friendly date/time
     FORM_FIELD_OVERRIDES = {
         models.DateField: {'widget': FriendlyDateInput},
         models.TimeField: {'widget': FriendlyTimeInput, 'form_class': FriendlyTimeField},
@@ -381,6 +382,12 @@ class BaseCompositeEditHandler(EditHandler):
     """
     _widget_overrides = None
 
+    def object_classnames(self):
+        try:
+            return "multi-field " + self.classname
+        except (AttributeError, TypeError):
+            return "multi-field"
+
     @classmethod
     def widget_overrides(cls):
         if cls._widget_overrides is None:
@@ -451,10 +458,11 @@ class BaseMultiFieldPanel(BaseCompositeEditHandler):
     template = "wagtailadmin/edit_handlers/multi_field_panel.html"
 
 
-def MultiFieldPanel(children, heading=""):
+def MultiFieldPanel(children, heading="", classname=None):
     return type('_MultiFieldPanel', (BaseMultiFieldPanel,), {
         'children': children,
         'heading': heading,
+        'classname': classname,
     })
 
 
@@ -720,5 +728,5 @@ Page.promote_panels = [
         FieldPanel('seo_title'),
         FieldPanel('show_in_menus'),
         FieldPanel('search_description'),
-    ], __('Common page configuration')),
+    ], ugettext_lazy('Common page configuration')),
 ]
