@@ -1,3 +1,15 @@
+var halloPlugins = {
+    'halloformat': {},
+    'halloheadings': {formatBlocks: ["p", "h2", "h3", "h4", "h5"]},
+    'hallolists': {},
+    'hallohr': {},
+    'halloreundo': {},
+    'hallowagtaillink': {},
+};
+function registerHalloPlugin(name, opts) {
+    halloPlugins[name] = (opts || {});
+}
+
 function makeRichTextEditable(id) {
     var input = $('#' + id);
     var richText = $('<div class="richtext"></div>').html(input.val());
@@ -19,17 +31,7 @@ function makeRichTextEditable(id) {
     richText.hallo({
         toolbar: 'halloToolbarFixed',
         toolbarcssClass: 'testy',
-        plugins: {
-            'halloformat': {},
-            'halloheadings': {formatBlocks: ["p", "h2", "h3", "h4", "h5"]},
-            'hallolists': {}, 
-            'hallohr': {},
-            'halloreundo': {},
-            'hallowagtailimage': {},
-            'hallowagtailembeds': {},
-            'hallowagtaillink': {},
-            'hallowagtaildoclink': {},
-        }
+        plugins: halloPlugins
     }).bind('hallomodified', function(event, data) {
         input.val(data.content);
         if (!removeStylingPending) {
@@ -55,7 +57,7 @@ function initDateChoosers(context) {
     $('input.friendly_date', context).datepicker({
         dateFormat: 'd M yy', constrainInput: false, /* showOn: 'button', */ firstDay: 1
     });
-    
+
     if(window.overrideDateInputFormat && window.overrideDateInputFormat !='') {
         $('input.localized_date', context).datepicker({
             dateFormat: window.overrideDateInputFormat, constrainInput: false, /* showOn: 'button', */ firstDay: 1
@@ -65,7 +67,7 @@ function initDateChoosers(context) {
             constrainInput: false, /* showOn: 'button', */ firstDay: 1
         });
     }
-    
+
 }
 function initFriendlyDateChooser(id) {
     $('#' + id).datepicker({
@@ -82,7 +84,7 @@ function initLocalizedDateChooser(id) {
             constrainInput: false, /* showOn: 'button', */ firstDay: 1
         });
     }
-    
+
 }
 
 function initTimeChoosers(context) {
@@ -134,7 +136,7 @@ function InlinePanel(opts) {
             $('#' + childId).slideUp(function() {
                 self.updateMoveButtonDisabledStates();
                 self.setHasContent();
-            });            
+            });
         });
         if (opts.canOrder) {
             $('#' + prefix + '-move-up').click(function() {
@@ -231,7 +233,7 @@ function InlinePanel(opts) {
                 $(fixPrefix('#id_' + opts.emptyChildFormPrefix + '-ORDER')).val(formCount);
             }
             self.updateMoveButtonDisabledStates();
-            
+
             opts.onAdd(fixPrefix);
         }
     });
@@ -239,8 +241,8 @@ function InlinePanel(opts) {
     return self;
 }
 
-function cleanForSlug(val){
-    if(URLify != undefined) { // Check to be sure that URLify function exists
+function cleanForSlug(val, useURLify){
+    if(URLify != undefined && useURLify !== false) { // Check to be sure that URLify function exists, and that we want to use it.
         return URLify(val, val.length);
     } else { // If not just do the "replace"
         return val.replace(/\s/g,"-").replace(/[^A-Za-z0-9\-]/g,"").toLowerCase();
@@ -261,8 +263,9 @@ function initSlugAutoPopulate(){
 }
 
 function initSlugCleaning(){
-    $('#id_slug').on('keyup blur', function(){
-        $(this).val(cleanForSlug($(this).val()));
+    $('#id_slug').blur(function(){
+        // if a user has just set the slug themselves, don't remove stop words etc, just illegal characters
+        $(this).val(cleanForSlug($(this).val(), false));
     });
 }
 
@@ -283,7 +286,26 @@ function initErrorDetection(){
     // now identify them on each tab
     for(var index in errorSections) {
         $('.tab-nav a[href=#'+ index +']').addClass('errors').attr('data-count', errorSections[index]);
-    }    
+    }
+}
+
+function initCollapsibleBlocks(){
+    $(".object.multi-field.collapsible").each(function(){
+        var $li = $(this);
+        var $fieldset = $li.find("fieldset");
+        if($li.hasClass("collapsed")){
+            $fieldset.hide();
+        }
+        $li.find("h2").click(function(){
+            if(!$li.hasClass("collapsed")){
+                $li.addClass("collapsed");
+                $fieldset.hide("slow");
+            }else{
+                $li.removeClass("collapsed");
+                $fieldset.show("show");
+            }
+        });
+    });
 }
 
 $(function() {
@@ -292,13 +314,14 @@ $(function() {
     initSlugAutoPopulate();
     initSlugCleaning();
     initErrorDetection();
+    initCollapsibleBlocks();
 
     $('.richtext [contenteditable="false"]').each(function() {
         insertRichTextDeleteControl(this);
     });
 
     /* Set up behaviour of preview button */
-    $('#action-preview').click(function() {
+    $('.action-preview').click(function() {
         var previewWindow = window.open($(this).data('placeholder'), $(this).data('windowname'));
 
         $.ajax({
@@ -328,5 +351,6 @@ $(function() {
                 previewWindow.document.close();
             }
         });
+        return false;
     });
 });
