@@ -5,6 +5,7 @@ from django.template import RequestContext
 from django.template.loader import render_to_string
 
 from wagtail.wagtailadmin import hooks
+from wagtail.wagtailadmin.forms import SearchForm
 
 from wagtail.wagtailcore.models import Page, PageRevision, UserPagePermissionsProxy
 
@@ -26,6 +27,7 @@ class SiteSummaryPanel(object):
             'total_pages': Page.objects.count() - 1,  # subtract 1 because the root node is not a real page
             'total_images': get_image_model().objects.count(),
             'total_docs': Document.objects.count(),
+            'search_form': SearchForm(),
         }, RequestContext(self.request))
 
 
