@@ -1,13 +1,12 @@
 from django.http import Http404, HttpResponse
 from django.shortcuts import render, redirect, get_object_or_404
 from django.core.exceptions import ValidationError, PermissionDenied
-from django.template.loader import render_to_string
-from django.template import RequestContext
 from django.contrib import messages
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.auth.decorators import permission_required
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
 from django.utils.translation import ugettext as _ 
+from django.views.decorators.vary import vary_on_headers
 
 from wagtail.wagtailadmin.edit_handlers import TabbedInterface, ObjectList
 from wagtail.wagtailadmin.forms import SearchForm
@@ -41,28 +40,6 @@ def index(request, parent_page_id=None):
     })
 
 
-@permission_required('wagtailadmin.access_admin')
-def select_type(request):
-    # Get the list of page types that can be created within the pages that currently exist
-    existing_page_types = ContentType.objects.raw("""
-        SELECT DISTINCT content_type_id AS id FROM wagtailcore_page
-    """)
-
-    all_page_types = sorted(get_page_types(), key=lambda pagetype: pagetype.name.lower())
-    page_types = set()
-    for content_type in existing_page_types:
-        allowed_subpage_types = content_type.model_class().clean_subpage_types()
-        for subpage_type in allowed_subpage_types:
-            subpage_content_type = ContentType.objects.get_for_model(subpage_type)
-
-            page_types.add(subpage_content_type)
-
-    return render(request, 'wagtailadmin/pages/select_type.html', {
-        'page_types': page_types,
-        'all_page_types': all_page_types
-    })
-
-
 @permission_required('wagtailadmin.access_admin')
 def add_subpage(request, parent_page_id):
     parent_page = get_object_or_404(Page, id=parent_page_id).specific
@@ -79,38 +56,6 @@ def add_subpage(request, parent_page_id):
     })
 
 
-@permission_required('wagtailadmin.access_admin')
-def select_location(request, content_type_app_name, content_type_model_name):
-    try:
-        content_type = ContentType.objects.get_by_natural_key(content_type_app_name, content_type_model_name)
-    except ContentType.DoesNotExist:
-        raise Http404
-
-    page_class = content_type.model_class()
-    # page_class must be a Page type and not some other random model
-    if not issubclass(page_class, Page):
-        raise Http404
-
-    # find all the valid locations (parent pages) where a page of the chosen type can be added
-    parent_pages = page_class.allowed_parent_pages()
-
-    if len(parent_pages) == 0:
-        # user cannot create a page of this type anywhere - fail with an error
-        messages.error(request, _("Sorry, you do not have access to create a page of type <em>'{0}'</em>.").format(content_type.name))
-        return redirect('wagtailadmin_pages_select_type')
-    elif len(parent_pages) == 1:
-        # only one possible location - redirect them straight there
-        messages.warning(request, _("Pages of this type can only be created as children of <em>'{0}'</em>. This new page will be saved there.").format(parent_pages[0].title))
-        return redirect('wagtailadmin_pages_create', content_type_app_name, content_type_model_name, parent_pages[0].id)
-    else:
-        # prompt them to select a location
-        return render(request, 'wagtailadmin/pages/select_location.html', {
-            'content_type': content_type,
-            'page_class': page_class,
-            'parent_pages': parent_pages,
-        })
-
-
 @permission_required('wagtailadmin.access_admin')
 def content_type_use(request, content_type_app_name, content_type_model_name):
     try:
@@ -118,15 +63,30 @@ def content_type_use(request, content_type_app_name, content_type_model_name):
     except ContentType.DoesNotExist:
         raise Http404
 
+    p = request.GET.get("p", 1)
+
     page_class = content_type.model_class()
 
     # page_class must be a Page type and not some other random model
     if not issubclass(page_class, Page):
         raise Http404
 
+    pages = page_class.objects.all()
+
+    paginator = Paginator(pages, 10)
+
+    try:
+        pages = paginator.page(p)
+    except PageNotAnInteger:
+        pages = paginator.page(1)
+    except EmptyPage:
+        pages = paginator.page(paginator.num_pages)
+
     return render(request, 'wagtailadmin/pages/content_type_use.html', {
-        'pages': page_class.objects.all(),
+        'pages': pages,
+        'app_name': content_type_app_name,
         'content_type': content_type,
+        'page_class': page_class,
     })
 
 
@@ -180,7 +140,7 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
                 page.live = False
                 page.has_unpublished_changes = True
 
-            parent_page.add_child(page)  # assign tree parameters - will cause page to be saved
+            parent_page.add_child(instance=page)  # assign tree parameters - will cause page to be saved
             page.save_revision(user=request.user, submitted_for_moderation=is_submitting)
 
             if is_publishing:
@@ -209,6 +169,7 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
         'page_class': page_class,
         'parent_page': parent_page,
         'edit_handler': edit_handler,
+        'display_modes': page.get_page_modes(),
     })
 
 
@@ -216,6 +177,8 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
 def edit(request, page_id):
     latest_revision = get_object_or_404(Page, id=page_id).get_latest_revision()
     page = get_object_or_404(Page, id=page_id).get_latest_revision_as_page()
+    parent = page.get_parent()
+
     page_perms = page.permissions_for_user(request.user)
     if not page_perms.can_edit():
         raise PermissionDenied
@@ -228,6 +191,14 @@ def edit(request, page_id):
     if request.POST:
         form = form_class(request.POST, request.FILES, instance=page)
 
+        # Stick an extra validator into the form to make sure that the slug is not already in use
+        def clean_slug(slug):
+            # Make sure the slug isn't already in use
+            if parent.get_children().filter(slug=slug).exclude(id=page_id).count() > 0:
+                raise ValidationError(_("This slug is already in use"))
+            return slug
+        form.fields['slug'].clean = clean_slug
+
         if form.is_valid():
             is_publishing = bool(request.POST.get('action-publish')) and page_perms.can_publish()
             is_submitting = bool(request.POST.get('action-submit'))
@@ -283,6 +254,7 @@ def edit(request, page_id):
         'page': page,
         'edit_handler': edit_handler,
         'errors_debug': errors_debug,
+        'display_modes': page.get_page_modes(),
     })
 
 
@@ -329,12 +301,19 @@ def preview_on_edit(request, page_id):
     if form.is_valid():
         form.save(commit=False)
 
-        # FIXME: passing the original request to page.serve is dodgy (particularly if page.serve has
-        # special treatment of POSTs). Ought to construct one that more or less matches what would be sent
-        # as a front-end GET request
+        # This view will generally be invoked as an AJAX request; as such, in the case of
+        # an error Django will return a plaintext response. This isn't what we want, since
+        # we will be writing the response back to an HTML page regardless of success or
+        # failure - as such, we strip out the X-Requested-With header to get Django to return
+        # an HTML error response
+        request.META.pop('HTTP_X_REQUESTED_WITH', None)
 
-        request.META.pop('HTTP_X_REQUESTED_WITH', None)  # Make this request appear to the page's serve method as a non-ajax one, as they will often implement custom behaviour for XHR
-        response = page.serve(request)
+        try:
+            display_mode = request.GET['mode']
+        except KeyError:
+            display_mode = page.get_page_modes()[0][0]
+
+        response = page.show_as_mode(display_mode)
 
         response['X-Wagtail-Preview'] = 'ok'
         return response
@@ -345,6 +324,7 @@ def preview_on_edit(request, page_id):
         response = render(request, 'wagtailadmin/pages/edit.html', {
             'page': page,
             'edit_handler': edit_handler,
+            'display_modes': page.get_page_modes(),
         })
         response['X-Wagtail-Preview'] = 'error'
         return response
@@ -369,10 +349,18 @@ def preview_on_create(request, content_type_app_name, content_type_model_name, p
     if form.is_valid():
         form.save(commit=False)
 
-        # FIXME: passing the original request to page.serve is dodgy (particularly if page.serve has
-        # special treatment of POSTs). Ought to construct one that more or less matches what would be sent
-        # as a front-end GET request
-        response = page.serve(request)
+        # This view will generally be invoked as an AJAX request; as such, in the case of
+        # an error Django will return a plaintext response. This isn't what we want, since
+        # we will be writing the response back to an HTML page regardless of success or
+        # failure - as such, we strip out the X-Requested-With header to get Django to return
+        # an HTML error response
+        request.META.pop('HTTP_X_REQUESTED_WITH', None)
+
+        try:
+            display_mode = request.GET['mode']
+        except KeyError:
+            display_mode = page.get_page_modes()[0][0]
+        response = page.show_as_mode(display_mode)
 
         response['X-Wagtail-Preview'] = 'ok'
         return response
@@ -386,6 +374,7 @@ def preview_on_create(request, content_type_app_name, content_type_model_name, p
             'page_class': page_class,
             'parent_page': parent_page,
             'edit_handler': edit_handler,
+            'display_modes': page.get_page_modes(),
         })
         response['X-Wagtail-Preview'] = 'error'
         return response
@@ -534,6 +523,7 @@ def get_page_edit_handler(page_class):
 
 
 @permission_required('wagtailadmin.access_admin')
+@vary_on_headers('X-Requested-With')
 def search(request):
     pages = []
     q = None
@@ -622,12 +612,7 @@ def preview_for_moderation(request, revision_id):
         return redirect('wagtailadmin_home')
 
     page = revision.as_page_object()
-    if not hasattr(request, 'userbar'):
-        request.userbar = []
-    request.userbar.append(
-        render_to_string('wagtailadmin/pages/_moderator_userbar.html', {
-            'revision': revision,
-        }, context_instance=RequestContext(request))
-    )
+
+    request.revision_id = revision_id
 
     return page.serve(request)
