@@ -1,5 +1,7 @@
 import sys
 import os
+from StringIO import StringIO
+from urlparse import urlparse
 
 from modelcluster.models import ClusterableModel
 
@@ -7,25 +9,22 @@ from django.db import models, connection, transaction
 from django.db.models import get_model, Q
 from django.http import Http404
 from django.core.cache import cache
+from django.core.handlers.wsgi import WSGIRequest
+from django.core.handlers.base import BaseHandler
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.auth.models import Group
 from django.conf import settings
 from django.template.response import TemplateResponse
 from django.utils.translation import ugettext_lazy as _
 
+from treebeard.mp_tree import MP_Node
+
 from wagtail.wagtailcore.util import camelcase_to_underscore
+from wagtail.wagtailcore.query import PageQuerySet
 
 from wagtail.wagtailsearch import Indexed, get_search_backend
 
 
-# hack to import our patched copy of treebeard at wagtail/vendor/django-treebeard -
-# based on http://stackoverflow.com/questions/17211078/how-to-temporarily-modify-sys-path-in-python
-treebeard_path = os.path.join(os.path.dirname(__file__), '..', 'vendor', 'django-treebeard')
-sys.path.insert(0, treebeard_path)
-from treebeard.mp_tree import MP_Node
-sys.path.pop(0)
-
-
 class SiteManager(models.Manager):
     def get_by_natural_key(self, hostname):
         return self.get(hostname=hostname)
@@ -128,6 +127,59 @@ def get_navigable_page_content_type_ids():
     return _NAVIGABLE_PAGE_CONTENT_TYPE_IDS
 
 
+class PageManager(models.Manager):
+    def get_query_set(self):
+        return PageQuerySet(self.model).order_by('path')
+
+    def live(self):
+        return self.get_query_set().live()
+
+    def not_live(self):
+        return self.get_query_set().not_live()
+
+    def page(self, other):
+        return self.get_query_set().page(other)
+
+    def not_page(self, other):
+        return self.get_query_set().not_page(other)
+
+    def descendant_of(self, other, inclusive=False):
+        return self.get_query_set().descendant_of(other, inclusive)
+
+    def not_descendant_of(self, other, inclusive=False):
+        return self.get_query_set().not_descendant_of(other, inclusive)
+
+    def child_of(self, other):
+        return self.get_query_set().child_of(other)
+
+    def not_child_of(self, other):
+        return self.get_query_set().not_child_of(other)
+
+    def ancestor_of(self, other, inclusive=False):
+        return self.get_query_set().ancestor_of(other, inclusive)
+
+    def not_ancestor_of(self, other, inclusive=False):
+        return self.get_query_set().not_ancestor_of(other, inclusive)
+
+    def parent_of(self, other):
+        return self.get_query_set().parent_of(other)
+
+    def not_parent_of(self, other):
+        return self.get_query_set().not_parent_of(other)
+
+    def sibling_of(self, other, inclusive=False):
+        return self.get_query_set().sibling_of(other, inclusive)
+
+    def not_sibling_of(self, other, inclusive=False):
+        return self.get_query_set().not_sibling_of(other, inclusive)
+
+    def type(self, model):
+        return self.get_query_set().type(model)
+
+    def not_type(self, model):
+        return self.get_query_set().not_type(model)
+
+
 class PageBase(models.base.ModelBase):
     """Metaclass for Page"""
     def __init__(cls, name, bases, dct):
@@ -138,10 +190,16 @@ class PageBase(models.base.ModelBase):
             # don't proceed with all this page type registration stuff
             return
 
+        # Add page manager
+        PageManager().contribute_to_class(cls, 'objects')
+
         if 'template' not in dct:
             # Define a default template path derived from the app name and model name
             cls.template = "%s/%s.html" % (cls._meta.app_label, camelcase_to_underscore(name))
 
+        if 'ajax_template' not in dct:
+            cls.ajax_template = None
+
         cls._clean_subpage_types = None  # to be filled in on first call to cls.clean_subpage_types
 
         if not dct.get('is_abstract'):
@@ -221,7 +279,7 @@ class Page(MP_Node, ClusterableModel, Indexed):
 
         return self.url_path
 
-    @transaction.commit_on_success  # ensure that changes are only committed when we have updated all descendant URL paths, to preserve consistency
+    @transaction.atomic  # ensure that changes are only committed when we have updated all descendant URL paths, to preserve consistency
     def save(self, *args, **kwargs):
         update_descendant_url_paths = False
 
@@ -244,6 +302,11 @@ class Page(MP_Node, ClusterableModel, Indexed):
 
         if update_descendant_url_paths:
             self._update_descendant_url_paths(old_url_path, new_url_path)
+
+        # Check if this is a root page of any sites and clear the 'wagtail_site_root_paths' key if so
+        if Site.objects.filter(root_page=self).exists():
+            cache.delete('wagtail_site_root_paths')
+
         return result
 
     def _update_descendant_url_paths(self, old_url_path, new_url_path):
@@ -254,6 +317,12 @@ class Page(MP_Node, ClusterableModel, Indexed):
                 SET url_path = %s || substr(url_path, %s)
                 WHERE path LIKE %s AND id <> %s
             """
+        elif connection.vendor == 'mysql':
+            update_statement = """
+                UPDATE wagtailcore_page
+                SET url_path= CONCAT(%s, substring(url_path, %s))
+                WHERE path LIKE %s AND id <> %s
+            """
         else:
             update_statement = """
                 UPDATE wagtailcore_page
@@ -325,10 +394,24 @@ class Page(MP_Node, ClusterableModel, Indexed):
 
         return revision.as_page_object()
 
+    def get_context(self, request):
+        return {
+            'self': self,
+            'request': request,
+        }
+
+    def get_template(self, request):
+        if request.is_ajax():
+            return self.ajax_template or self.template
+        else:
+            return self.template
+
     def serve(self, request):
-        return TemplateResponse(request, self.template, {
-            'self': self
-        })
+        return TemplateResponse(
+            request, 
+            self.get_template(request), 
+            self.get_context(request)
+        )
 
     def is_navigable(self):
         """
@@ -415,7 +498,7 @@ class Page(MP_Node, ClusterableModel, Indexed):
                     if model:
                         res.append(model)
                     else:
-                        raise NameError(_("name '%s' (used in subpage_types list) is not defined.").format(page_type))
+                        raise NameError(_("name '{0}' (used in subpage_types list) is not defined.").format(page_type))
 
                 else:
                     # assume it's already a model class
@@ -463,7 +546,7 @@ class Page(MP_Node, ClusterableModel, Indexed):
         """
         return (not self.live) and (not self.get_descendants().filter(live=True).exists())
 
-    @transaction.commit_on_success  # only commit when all descendants are properly updated
+    @transaction.atomic  # only commit when all descendants are properly updated
     def move(self, target, pos=None):
         """
         Extension to the treebeard 'move' method to ensure that url_path is updated too.
@@ -484,6 +567,83 @@ class Page(MP_Node, ClusterableModel, Indexed):
         user_perms = UserPagePermissionsProxy(user)
         return user_perms.for_page(self)
 
+    def dummy_request(self):
+        """
+        Construct a HttpRequest object that is, as far as possible, representative of ones that would
+        receive this page as a response. Used for previewing / moderation and any other place where we
+        want to display a view of this page in the admin interface without going through the regular
+        page routing logic.
+        """
+        url = self.full_url
+        if url:
+            url_info = urlparse(url)
+            hostname = url_info.hostname
+            path = url_info.path
+            port = url_info.port or 80
+        else:
+            hostname = 'example.com'
+            path = '/'
+            port = 80
+
+        request = WSGIRequest({
+            'REQUEST_METHOD': 'GET',
+            'PATH_INFO': path,
+            'SERVER_NAME': hostname,
+            'SERVER_PORT': port,
+            'wsgi.input': StringIO(),
+        })
+
+        # Apply middleware to the request - see http://www.mellowmorning.com/2011/04/18/mock-django-request-for-testing/
+        handler = BaseHandler()
+        handler.load_middleware()
+        for middleware_method in handler._request_middleware:
+            if middleware_method(request):
+                raise Exception("Couldn't create request mock object - "
+                                "request middleware returned a response")
+        return request
+
+    def get_page_modes(self):
+        """
+        Return a list of (internal_name, display_name) tuples for the modes in which
+        this page can be displayed for preview/moderation purposes. Ordinarily a page
+        will only have one display mode, but subclasses of Page can override this -
+        for example, a page containing a form might have a default view of the form,
+        and a post-submission 'thankyou' page
+        """
+        return [('', 'Default')]
+
+    def show_as_mode(self, mode_name):
+        """
+        Given an internal name from the get_page_modes() list, return an HTTP response
+        indicative of the page being viewed in that mode. By default this passes a
+        dummy request into the serve() mechanism, ensuring that it matches the behaviour
+        on the front-end; subclasses that define additional page modes will need to
+        implement alternative logic to serve up the appropriate view here.
+        """
+        return self.serve(self.dummy_request())
+
+    def get_static_site_paths(self):
+        """
+        This is a generator of URL paths to feed into a static site generator
+        Override this if you would like to create static versions of subpages
+        """
+        # Yield paths for this page
+        yield '/'
+
+        # Yield paths for child pages
+        for child in self.get_children().live():
+            for path in child.specific.get_static_site_paths():
+                yield '/' + child.slug + path
+
+    def get_ancestors(self, inclusive=False):
+        return Page.objects.ancestor_of(self, inclusive)
+
+    def get_descendants(self, inclusive=False):
+        return Page.objects.descendant_of(self, inclusive)
+
+    def get_siblings(self, inclusive=True):
+        return Page.objects.sibling_of(self, inclusive)
+
 
 def get_navigation_menu_items():
     # Get all pages that appear in the navigation menu: ones which have children,
@@ -491,17 +651,9 @@ def get_navigation_menu_items():
     # or are at the top-level (this rule required so that an empty site out-of-the-box has a working menu)
     navigable_content_type_ids = get_navigable_page_content_type_ids()
     if navigable_content_type_ids:
-        pages = Page.objects.raw("""
-            SELECT * FROM wagtailcore_page
-            WHERE numchild > 0 OR content_type_id IN %s OR depth = 2
-            ORDER BY path
-        """, [tuple(navigable_content_type_ids)])
+        pages = Page.objects.filter(Q(content_type__in=navigable_content_type_ids)|Q(depth=2)|Q(numchild__gt=0)).order_by('path')
     else:
-        pages = Page.objects.raw("""
-            SELECT * FROM wagtailcore_page
-            WHERE numchild > 0 OR depth = 2
-            ORDER BY path
-        """)
+        pages = Page.objects.filter(Q(depth=2)|Q(numchild__gt=0)).order_by('path')
 
     # Turn this into a tree structure:
     #     tree_node = (page, children)
@@ -647,6 +799,37 @@ class UserPagePermissionsProxy(object):
         permission to perform specific tasks on the given page"""
         return PagePermissionTester(self, page)
 
+    def editable_pages(self):
+        """Return a queryset of the pages that this user has permission to edit"""
+        # Deal with the trivial cases first...
+        if not self.user.is_active:
+            return Page.objects.none()
+        if self.user.is_superuser:
+            return Page.objects.all()
+
+        # Translate each of the user's permission rules into a Q-expression
+        q_expressions = []
+        for perm in self.permissions:
+            if perm.permission_type == 'add':
+                # user has edit permission on any subpage of perm.page
+                # (including perm.page itself) that is owned by them
+                q_expressions.append(
+                    Q(path__startswith=perm.page.path, owner=self.user)
+                )
+            elif perm.permission_type == 'edit':
+                # user has edit permission on any subpage of perm.page
+                # (including perm.page itself) regardless of owner
+                q_expressions.append(
+                    Q(path__startswith=perm.page.path)
+                )
+
+        if q_expressions:
+            all_rules = q_expressions[0]
+            for expr in q_expressions[1:]:
+                all_rules = all_rules | expr
+            return Page.objects.filter(all_rules)
+        else:
+            return Page.objects.none()
 
 class PagePermissionTester(object):
     def __init__(self, user_perms, page):
