@@ -1,7 +1,7 @@
 from django import template
 from django.utils.safestring import mark_safe
 
-from wagtail.wagtailembeds.embeds import get_embed
+from wagtail.wagtailembeds import get_embed
 
 
 register = template.Library()
@@ -10,9 +10,12 @@ register = template.Library()
 @register.filter
 def embed(url, max_width=None):
     embed = get_embed(url, max_width=max_width)
-    if embed is not None:
-        return mark_safe(embed.html)
-    else:
+    try:
+        if embed is not None:
+            return mark_safe(embed.html)
+        else:
+            return ''
+    except:
         return ''
 
 
