@@ -1,11 +1,10 @@
 import StringIO
 import os.path
 
-import PIL.Image
 from taggit.managers import TaggableManager
 
 from django.core.files import File
-from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist, ValidationError
+from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.db import models
 from django.db.models.signals import pre_delete
 from django.dispatch.dispatcher import receiver
@@ -14,8 +13,11 @@ from django.utils.html import escape
 from django.conf import settings
 from django.utils.translation import ugettext_lazy  as _
 
+from unidecode import unidecode
+
 from wagtail.wagtailadmin.taggable import TagSearchable
-from wagtail.wagtailimages import image_ops
+from wagtail.wagtailimages.backends import get_image_backend
+from .utils import validate_image_format
 
 
 class AbstractImage(models.Model, TagSearchable):
@@ -25,20 +27,16 @@ class AbstractImage(models.Model, TagSearchable):
         folder_name = 'original_images'
         filename = self.file.field.storage.get_valid_name(filename)
 
+        # do a unidecode in the filename and then
         # replace non-ascii characters in filename with _ , to sidestep issues with filesystem encoding
-        filename = "".join((i if ord(i) < 128 else '_') for i in filename)
+        filename = "".join((i if ord(i) < 128 else '_') for i in unidecode(filename))
 
         while len(os.path.join(folder_name, filename)) >= 95:
             prefix, dot, extension = filename.rpartition('.')
             filename = prefix[:-1] + dot + extension
         return os.path.join(folder_name, filename)
 
-    def file_extension_validator(ffile):
-        extension = ffile.name.split(".")[-1].lower()
-        if extension not in ["gif", "jpg", "jpeg", "png"]:
-            raise ValidationError(_("Not a valid image format. Please use a gif, jpeg or png file instead."))
-
-    file = models.ImageField(verbose_name=_('File'), upload_to=get_upload_to, width_field='width', height_field='height', validators=[file_extension_validator])
+    file = models.ImageField(verbose_name=_('File'), upload_to=get_upload_to, width_field='width', height_field='height', validators=[validate_image_format])
     width = models.IntegerField(editable=False)
     height = models.IntegerField(editable=False)
     created_at = models.DateTimeField(auto_now_add=True)
@@ -68,7 +66,11 @@ class AbstractImage(models.Model, TagSearchable):
             rendition = self.renditions.get(filter=filter)
         except ObjectDoesNotExist:
             file_field = self.file
-            generated_image_file = filter.process_image(file_field.file)
+
+			# If we have a backend attribute then pass it to process
+			# image - else pass 'default'
+            backend_name = getattr(self, 'backend', 'default')
+            generated_image_file = filter.process_image(file_field.file, backend_name=backend_name)
 
             rendition, created = self.renditions.get_or_create(
                 filter=filter, defaults={'file': generated_image_file})
@@ -143,11 +145,11 @@ class Filter(models.Model):
     spec = models.CharField(max_length=255, db_index=True)
 
     OPERATION_NAMES = {
-        'max': image_ops.resize_to_max,
-        'min': image_ops.resize_to_min,
-        'width': image_ops.resize_to_width,
-        'height': image_ops.resize_to_height,
-        'fill': image_ops.resize_to_fill,
+        'max': 'resize_to_max',
+        'min': 'resize_to_min',
+        'width': 'resize_to_width',
+        'height': 'resize_to_height',
+        'fill': 'resize_to_fill',
     }
 
     def __init__(self, *args, **kwargs):
@@ -156,12 +158,12 @@ class Filter(models.Model):
 
     def _parse_spec_string(self):
         # parse the spec string, which is formatted as (method)-(arg),
-        # and save the results to self.method and self.method_arg
+        # and save the results to self.method_name and self.method_arg
         try:
-            (method_name, method_arg_string) = self.spec.split('-')
-            self.method = Filter.OPERATION_NAMES[method_name]
+            (method_name_simple, method_arg_string) = self.spec.split('-')
+            self.method_name = Filter.OPERATION_NAMES[method_name_simple]
 
-            if method_name in ('max', 'min', 'fill'):
+            if method_name_simple in ('max', 'min', 'fill'):
                 # method_arg_string is in the form 640x480
                 (width, height) = [int(i) for i in method_arg_string.split('x')]
                 self.method_arg = (width, height)
@@ -172,24 +174,33 @@ class Filter(models.Model):
         except (ValueError, KeyError):
             raise ValueError("Invalid image filter spec: %r" % self.spec)
 
-    def process_image(self, input_file):
+    def process_image(self, input_file, backend_name='default'):
         """
         Given an input image file as a django.core.files.File object,
         generate an output image with this filter applied, returning it
         as another django.core.files.File object
         """
+        
+        backend = get_image_backend(backend_name)
+        
         if not self.method:
             self._parse_spec_string()
-
-        input_file.open()
-        image = PIL.Image.open(input_file)
+        
+        # If file is closed, open it
+        input_file.open('rb')
+        image = backend.open_image(input_file)
         file_format = image.format
+        
+        method = getattr(backend, self.method_name)
 
-        # perform the resize operation
-        image = self.method(image, self.method_arg)
+        image = method(image, self.method_arg)
 
         output = StringIO.StringIO()
-        image.save(output, file_format)
+        backend.save_image(image, output, file_format)
+        
+        # and then close the input file
+        input_file.close()
+        
 
         # generate new filename derived from old one, inserting the filter spec string before the extension
         input_filename_parts = os.path.basename(input_file.name).split('.')
@@ -199,7 +210,7 @@ class Filter(models.Model):
         output_filename = '.'.join(output_filename_parts)
 
         output_file = File(output, name=output_filename)
-        input_file.close()
+        
 
         return output_file
 
